import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Date;
import java.text.DateFormat;

/**
Beispielprogramm zur Variableninitialisierung sowie static und final
@author Dipl.-Ing. Michael Sonntag
@version 1.0
*/
class Initializers 
{
    /** Ist ffentlich bekannt, kann nicht verndert werden, gilt fr ALLE Buffer. Typische Konstante */
    public static final int N_MIN=-10;
    public static final int N_MAX=10;

    /* Ist im Paket bekannt, kann nicht verndert werden, gilt fr ALLE Buffer. Typische Konstante */
    static final int BUFLEN=5;

    /* Ist nur lokal bekannt, kann verndert werden, gilt nur fr JEWEILIGES Objekt */
    private int curBufferPos;

    /* Ganz normale Variablen */
    int[] buffer1=null;
    int[] buffer2=new int[BUFLEN];
    boolean full;

    /* Existiert nur einmal; ist fr alle Objekte gleich! */
    private static int bufferCount;

    /* Objekt kann nicht mehr verndert werden, nur mehr die internen Werte */
    final GregorianCalendar cal=new GregorianCalendar();

    /* Objekt kann nicht mehr verndert werden, nur mehr die internen Werte; Gilt fr alle Objekte dieser Klasse! */
    /* KEIN new, da eine neuen Instanz ber statische Methode erzeugt wird */
    static final DateFormat df=DateFormat.getDateTimeInstance();
    

public Initializers (int i)
{
    curBufferPos=i;    // Initialisierung im Konstruktor; nur fr Instanzvariablen
    System.out.println("Derzeitige Bufferposition: "+curBufferPos); // Geht, da jetzt ein Objekt dazu existiert (=this)!
}

public void doIt()
{
    cal.setTime(new Date());    // Geht, da cal das selbe Objekt bleibt; nur innere Werte anders!
    System.out.println("Zeit: "+df.format(cal.getTime()));
//    cal=new GregorianCalendar();    Geht nicht, da cal final!

    // df kann nicht verndert werden, da es final ist. Es existieren keine Methoden zur nderung der Formatierung, daher neues Objekt ntig
    DateFormat fulldf=DateFormat.getDateTimeInstance(DateFormat.FULL,DateFormat.FULL);
    cal.add(Calendar.HOUR,1);
    System.out.println("Zeit: "+fulldf.format(cal.getTime()));
    // Statische Variablen knnen immer und berall verndert werden
    bufferCount++;
}

static public void main(String args[])
{
    System.out.println("Programmstart\n");
    
//    N_MIN=-8; Geht nicht, da ein statischer Wert nicht verndert werden kann
//    System.out.println("Derzeitige Bufferposition: "+curBufferPos); Geht nicht, da kein Objekt dazu existiert!

    // Statischer Wert existiert auch ohne Objekt
    System.out.println("Bufferanzahl: "+bufferCount);

    Initializers ini1=new Initializers(1);
    Initializers ini2=new Initializers(0);
    // Instanzvariable -> jedes Objekt ein eigenes
    if(ini1.cal==ini2.cal)
        System.out.println("Calendar gleich");
    else
        System.out.println("Calendar ungleich");
    // static -> Gleich fr alle Objekte
    if(ini1.df==ini2.df)
        System.out.println("DateFormat gleich");
    else
        System.out.println("DateFormat ungleich");
    // Objekt bearbeiten
    ini1.doIt();
    // Statischen Werte berprfen: nderungen in einer Klassenmethode bleiben auch auerhalb erhalten
    System.out.println("Bufferanzahl: "+bufferCount);
    System.out.println("\nProgrammende");
}

}

